/*
 * orthodisp.h
 *
 *  Created on: Jun 25, 2009
 *      Author: jb
 */

#ifndef ORTHODISP_H_
#define ORTHODISP_H_

#ifndef DEBUG
#define DEBUG 0
#endif

#include <stdio.h>
#include <complex.h>

typedef struct {
	int id;
	char name[13];
	double rho;                     // in g / cm^3
	double E1, E2, E3;              // in GPa
	double G12, G23, G31;           // in GPa
	double nu12, nu23, nu31;
} material;

typedef struct {
	int size;
	material *mat;
} materialDB;

typedef struct {
	material *mat;
	double thickness;
	double angle;
	double C[6][6];
} layer;

typedef struct {
	int numlayers;
	double thickness;
	layer *stacking;
} laminate;

/* bulk.c */
void print_ortho_bulk_wave(char *fname, material mat);

/* christoffel.c */
void get_ortho_christoffel_tensor(material mat, double no[3], double lambda[3][3]);
void get_polarisation (double c[6][6], double rhocp2, double sintheta, complex double lz[6], complex double P[3][6], complex double D[3][6]);
void get_char_eqn_coeff_monocl(double c[6][6], double rhocp2, double sin2, double a[4]);
void get_char_eqn_coeff_ortho(double c[6][6], double rhocp2, double sin2, double a[4]);

/* io.c */
void parse_input_file(char *inpfn, char *logfn, materialDB *matdb, laminate *lam, double *sintheta, double *phi, double *cp_start, double *cp_end, double *cp_incr, int *cp_steps, double *w_start, double *w_end, double *w_incr, int *w_steps);
int count_materials (char *filename);
int read_materials(char *filename, material *matdb);
int read_range(char * filename, double *cp0, double *cp1, int *cpinc, double *w0,  double *w1,  int *winc);
int read_waveprop(char *filename, double *sintheta, double *phi);
int count_layers(char *filename);
int read_layers(char *filename, material *matdb, int nmat, double phi, layer *laminate);
void debugmsg(int debuglvl, FILE *f, char *format, ...);
void print_d_matrix(FILE *f, double *M, int r, int c);
void print_c_matrix(FILE *f, complex double *M, int r, int c);

/* math.c */
complex double clapack_complex_det(complex double *C, int size);
int clapack_complex_inverse(complex double *C, complex double *CI, int size);
int clapack_complex_eigensystem(complex double *A, complex double *B, int size, complex double *lambda, complex double *vl, complex double *vr);
int sign(double n);
void roots_of_poly0246(double a[4], complex double lz[6]);
void roots_of_poly024_ortho(double a[4], complex double lz[6]);
void matmul(double *a, double *b, double *c, int size);
void complex_matmul(complex double *a, complex double *b, complex double *c, int size);
void complex_vecmul(complex double *a, complex double *b, complex double *c, int size);
void transpose(double *a, double *b, int size);
void find_zeros_c(complex double *c, int size, int *s);
void reduce_singular_soe(complex double *M, int size, int r, int c, complex double *LHS, complex double *Rhs);

/* material.c */
void get_layer_prop(layer *l, double *rho, double *d, double C[6][6]);
void set_layer_C(layer *l);
void get_ortho_stiff_tensor(material mat, double c[3][3][3][3]);
void get_test_ortho_stiff(double C[6][6]);
void get_ortho_stiff(material mat, double Cortho[6][6]);
void get_trans(double phi, double T[6][6]);
void get_trans_i(double phi, double T[6][6]);
int is_monocl(double C[6][6]);

/* wang.c */
void wang_total_stiffness_matrix (layer *laminate, int nl, double cp, double waveno, double sintheta, complex double KT[6][6]);
void wang_layer_stiffness_matrix (double c[6][6], double rhocp2, double k, double d, double sintheta, complex double lz[6], complex double K[6][6]);
void wang_add_layer_to_stiffness_matrix (double c[6][6], double rhocp2, double k, double d, double sintheta, complex double lz[6], complex double KA[6][6], complex double KB[6][6]);
void wang_surface_displacement(complex double KT[6][6], complex double displ_u[3], complex double displ_l[3]);
void wang_surface_displacement2(complex double KT[6][6], int u_preset, int r_delete, complex double displ_u[3], complex double displ_l[3]);
void find_cps(laminate *lam, double w, double sintheta, int *sgn, int size, double cp_start, double cp_incr, double *cp);

#endif /* ORTHODISP_H_ */
